from typing import Dict, Any
from csle_base.json_serializable import JSONSerializable


class Action(JSONSerializable):
    """
    DTO representing an action in a simulation environment
    """

    def __init__(self, id: int, descr: str):
        """
        Initializes the DTO

        :param id: the id of the action
        :param descr: the description of the action
        """
        self.id = id
        self.descr = descr

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "Action":
        """
        Converts a dict representation of the DTO into an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = Action(
            id=d["id"], descr=d["descr"]
        )
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        :return: a dict representation of the DTO
        """
        d: Dict[str, Any] = {}
        d["id"] = self.id
        d["descr"] = self.descr
        return d

    def __str__(self) -> str:
        """
        :return: a string representation of the DTO
        """
        return f"id:{self.id}, descr:{self.descr}"

    @staticmethod
    def from_json_file(json_file_path: str) -> "Action":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return Action.from_dict(json.loads(json_str))



from typing import List, Dict, Any
from csle_common.dao.emulation_config.ovs_switch_config import OvsSwitchConfig
from csle_base.json_serializable import JSONSerializable


class OVSConfig(JSONSerializable):
    """
    DTO containing the configuration of OVS in an emulation
    """

    def __init__(self, switch_configs: List[OvsSwitchConfig]):
        """
        Initializes the DTO

        :param switch_configs: list of OVS switch configs
        """
        self.switch_configs = switch_configs

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "OVSConfig":
        """
        Converts a dict representation to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = OVSConfig(switch_configs=list(map(lambda x: OvsSwitchConfig.from_dict(x), d["switch_configs"])))
        return obj

    def to_dict(self) -> Dict[str, Any]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d = {}
        d["switch_configs"] = list(map(lambda x: x.to_dict(), self.switch_configs))
        return d

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return f"switch_configs:{list(map(lambda x: str(x), self.switch_configs))}"

    @staticmethod
    def from_json_file(json_file_path: str) -> "OVSConfig":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return OVSConfig.from_dict(json.loads(json_str))

    def copy(self) -> "OVSConfig":
        """
        :return: a copy of the DTO
        """
        return OVSConfig.from_dict(self.to_dict())

    def create_execution_config(self, ip_first_octet: int, physical_servers: List[str]) -> "OVSConfig":
        """
        Creates a new config for an execution

        :param ip_first_octet: the first octet of the IP of the new execution
        :param physical_servers: the list of physical servers of the execution
        :return: the new config
        """
        config = self.copy()
        config.switch_configs = list(map(lambda x: x.create_execution_config(
            ip_first_octet=ip_first_octet, physical_servers=physical_servers), config.switch_configs))
        return config



from typing import List
import threading
import time
from confluent_kafka import Consumer, KafkaError, KafkaException
from csle_common.logging.log import Logger
from csle_common.dao.emulation_action.defender.emulation_defender_action import EmulationDefenderAction
import csle_collector.constants.constants as collector_constants


class DefenderActionsConsumerThread(threading.Thread):
    """
    Thread that polls the kafka to get the latest defender actions
    """

    def __init__(self, kafka_server_ip: str, kafka_port: int,
                 defender_actions: List[EmulationDefenderAction], auto_offset_reset: str = "latest") -> None:
        """
        Initializes the thread

        :param kafka_server_ip: the ip of the kafka server
        :param kafka_port: the port of the kafka server
        :param defender_actions: the defender actions to update
        :param auto_offset_reset: the offset for kafka to start reading from
        """
        threading.Thread.__init__(self)
        self.running = True
        self.kafka_server_ip = kafka_server_ip
        self.kafka_port = kafka_port
        self.ts = time.time()
        self.auto_offset_reset = auto_offset_reset
        self.kafka_conf = {
            collector_constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: f"{self.kafka_server_ip}:{self.kafka_port}",
            collector_constants.KAFKA.GROUP_ID_PROPERTY: f"defender_actions_consumer_thread_{self.ts}",
            collector_constants.KAFKA.AUTO_OFFSET_RESET_PROPERTY: auto_offset_reset
        }
        self.consumer = Consumer(**self.kafka_conf)
        self.consumer.subscribe([collector_constants.KAFKA_CONFIG.DEFENDER_ACTIONS_TOPIC_NAME])
        self.defender_actions = defender_actions

    def run(self) -> None:
        """
        Runs the thread

        :return: None
        """
        while self.running:
            msg = self.consumer.poll(timeout=5.0)
            if msg is not None:
                if msg.error():
                    if msg.error().code() == KafkaError._PARTITION_EOF:
                        Logger.__call__().get_logger.warning(
                            f"reached end of partition: {msg.topic(), msg.partition(), msg.offset()}")
                    elif msg.error():
                        raise KafkaException(msg.error())
                else:
                    self.defender_actions.append(EmulationDefenderAction.from_kafka_record(
                        record=msg.value().decode()))
        self.consumer.close()



from typing import Dict, Any, Union
from csle_base.json_serializable import JSONSerializable


class FiveGConfig(JSONSerializable):
    """
    Represents the configuration of the 5G services and managers in a CSLE emulation
    """

    def __init__(self, five_g_core_manager_log_file: str, five_g_core_manager_log_dir: str,
                 five_g_core_manager_max_workers: int,
                 five_g_cu_manager_log_file: str, five_g_cu_manager_log_dir: str,
                 five_g_cu_manager_max_workers: int,
                 five_g_du_manager_log_file: str, five_g_du_manager_log_dir: str,
                 five_g_du_manager_max_workers: int,
                 time_step_len_seconds: int = 15, five_g_core_manager_port: int = 50052,
                 five_g_cu_manager_port: int = 50053, five_g_du_manager_port: int = 50054,
                 version: str = "0.0.1") -> None:
        """
        Initializes the DTO

        :param time_step_len_seconds: the length of a time-step (period for logging)
        :param version: the version
        :param five_g_core_manager_port: the GRPC port of the 5G core manager
        :param five_g_core_manager_log_file: Log file of the 5G core manager
        :param five_g_core_manager_log_dir: Log dir of the 5G core manager
        :param five_g_core_manager_max_workers: Max GRPC workers of the 5G core manager
        :param five_g_cu_manager_port: the GRPC port of the 5G cu manager
        :param five_g_cu_manager_log_file: Log file of the 5G cu manager
        :param five_g_cu_manager_log_dir: Log dir of the 5G cu manager
        :param five_g_cu_manager_max_workers: Max GRPC workers of the 5G cu manager
        :param five_g_du_manager_port: the GRPC port of the 5G du manager
        :param five_g_du_manager_log_file: Log file of the 5G du manager
        :param five_g_du_manager_log_dir: Log dir of the 5G du manager
        :param five_g_du_manager_max_workers: Max GRPC workers of the 5G du manager
        """
        self.time_step_len_seconds = time_step_len_seconds
        self.version = version

        self.five_g_core_manager_port = five_g_core_manager_port
        self.five_g_core_manager_log_file = five_g_core_manager_log_file
        self.five_g_core_manager_log_dir = five_g_core_manager_log_dir
        self.five_g_core_manager_max_workers = five_g_core_manager_max_workers

        self.five_g_cu_manager_port = five_g_cu_manager_port
        self.five_g_cu_manager_log_file = five_g_cu_manager_log_file
        self.five_g_cu_manager_log_dir = five_g_cu_manager_log_dir
        self.five_g_cu_manager_max_workers = five_g_cu_manager_max_workers

        self.five_g_du_manager_port = five_g_du_manager_port
        self.five_g_du_manager_log_file = five_g_du_manager_log_file
        self.five_g_du_manager_log_dir = five_g_du_manager_log_dir
        self.five_g_du_manager_max_workers = five_g_du_manager_max_workers

    @staticmethod
    def from_dict(d: Dict[str, Any]) -> "FiveGConfig":
        """
        Converts a dict representation to an instance

        :param d: the dict to convert
        :return: the created instance
        """
        obj = FiveGConfig(
            time_step_len_seconds=d["time_step_len_seconds"], version=d["version"],
            five_g_core_manager_log_file=d["five_g_core_manager_log_file"],
            five_g_core_manager_log_dir=d["five_g_core_manager_log_dir"],
            five_g_core_manager_max_workers=d["five_g_core_manager_max_workers"],
            five_g_core_manager_port=d["five_g_core_manager_port"],
            five_g_cu_manager_log_file=d["five_g_cu_manager_log_file"],
            five_g_cu_manager_log_dir=d["five_g_cu_manager_log_dir"],
            five_g_cu_manager_max_workers=d["five_g_cu_manager_max_workers"],
            five_g_cu_manager_port=d["five_g_cu_manager_port"],
            five_g_du_manager_log_file=d["five_g_du_manager_log_file"],
            five_g_du_manager_log_dir=d["five_g_du_manager_log_dir"],
            five_g_du_manager_max_workers=d["five_g_du_manager_max_workers"],
            five_g_du_manager_port=d["five_g_du_manager_port"]
        )
        return obj

    def to_dict(self) -> Dict[str, Union[str, int]]:
        """
        Converts the object to a dict representation

        :return: a dict representation of the object
        """
        d: Dict[str, Union[str, int]] = {}
        d["time_step_len_seconds"] = self.time_step_len_seconds
        d["version"] = self.version
        d["five_g_core_manager_log_file"] = self.five_g_core_manager_log_file
        d["five_g_core_manager_log_dir"] = self.five_g_core_manager_log_dir
        d["five_g_core_manager_max_workers"] = self.five_g_core_manager_max_workers
        d["five_g_core_manager_port"] = self.five_g_core_manager_port
        d["five_g_cu_manager_log_file"] = self.five_g_cu_manager_log_file
        d["five_g_cu_manager_log_dir"] = self.five_g_cu_manager_log_dir
        d["five_g_cu_manager_max_workers"] = self.five_g_cu_manager_max_workers
        d["five_g_cu_manager_port"] = self.five_g_cu_manager_port
        d["five_g_du_manager_log_file"] = self.five_g_du_manager_log_file
        d["five_g_du_manager_log_dir"] = self.five_g_du_manager_log_dir
        d["five_g_du_manager_max_workers"] = self.five_g_du_manager_max_workers
        d["five_g_du_manager_port"] = self.five_g_du_manager_port
        return d

    def __str__(self) -> str:
        """
        :return: a string representation of the object
        """
        return (f"time_step_len_seconds: {self.time_step_len_seconds}, version: {self.version}, "
                f"five_g_core_manager_log_file: {self.five_g_core_manager_log_file}, "
                f"five_g_core_manager_log_dir: {self.five_g_core_manager_log_dir}, "
                f"five_g_core_manager_max_workers: {self.five_g_core_manager_max_workers}, "
                f"five_g_core_manager_port: {self.five_g_core_manager_port}"
                f"five_g_cu_manager_log_file: {self.five_g_cu_manager_log_file}, "
                f"five_g_cu_manager_log_dir: {self.five_g_cu_manager_log_dir}, "
                f"five_g_cu_manager_max_workers: {self.five_g_cu_manager_max_workers}, "
                f"five_g_cu_manager_port: {self.five_g_cu_manager_port}"
                f"five_g_du_manager_log_file: {self.five_g_du_manager_log_file}, "
                f"five_g_du_manager_log_dir: {self.five_g_du_manager_log_dir}, "
                f"five_g_du_manager_max_workers: {self.five_g_du_manager_max_workers}, "
                f"five_g_du_manager_port: {self.five_g_du_manager_port}")

    @staticmethod
    def from_json_file(json_file_path: str) -> "FiveGConfig":
        """
        Reads a json file and converts it to a DTO

        :param json_file_path: the json file path
        :return: the converted DTO
        """
        import io
        import json
        with io.open(json_file_path, 'r') as f:
            json_str = f.read()
        return FiveGConfig.from_dict(json.loads(json_str))

    def copy(self) -> "FiveGConfig":
        """
        :return: a copy of the DTO
        """
        return FiveGConfig.from_dict(self.to_dict())

    def create_execution_config(self, ip_first_octet: int) -> "FiveGConfig":
        """
        Creates a new config for an execution

        :param ip_first_octet: the first octet of the IP of the new execution
        :return: the new config
        """
        config = self.copy()
        return config

    @staticmethod
    def schema() -> "FiveGConfig":
        """
        :return: get the schema of the DTO
        """
        return FiveGConfig(
            version="0.0.1", time_step_len_seconds=15,
            five_g_core_manager_log_file="five_g_core_manager.log",
            five_g_core_manager_port=50052,
            five_g_core_manager_log_dir="/",
            five_g_core_manager_max_workers=10,
            five_g_cu_manager_log_file="five_g_cu_manager.log",
            five_g_cu_manager_port=50052,
            five_g_cu_manager_log_dir="/",
            five_g_cu_manager_max_workers=10,
            five_g_du_manager_log_file="five_g_du_manager.log",
            five_g_du_manager_port=50052,
            five_g_du_manager_log_dir="/",
            five_g_du_manager_max_workers=10
        )
